<?php

namespace Cas\Result\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Cas\Result\Http\Requests\ResultRequest;
use Cas\Result\Interfaces\ResultRepositoryInterface;
use Cas\Result\Models\Result;

/**
 * Resource controller class for result.
 */
class ResultResourceController extends BaseController
{

    /**
     * Initialize result resource controller.
     *
     * @param type ResultRepositoryInterface $result
     *
     * @return null
     */
    public function __construct(ResultRepositoryInterface $result)
    {
        parent::__construct();
        $this->repository = $result;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Cas\Result\Repositories\Criteria\ResultResourceCriteria::class);
    }

    /**
     * Display a list of result.
     *
     * @return Response
     */
    public function index(ResultRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Cas\Result\Repositories\Presenter\ResultPresenter::class)
                ->$function();
        }

        $results = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('result::result.names'))
            ->view('result::result.index', true)
            ->data(compact('results', 'view'))
            ->output();
    }

    /**
     * Display result.
     *
     * @param Request $request
     * @param Model   $result
     *
     * @return Response
     */
    public function show(ResultRequest $request, Result $result)
    {

        if ($result->exists) {
            $view = 'result::result.show';
        } else {
            $view = 'result::result.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('result::result.name'))
            ->data(compact('result'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new result.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ResultRequest $request)
    {

        $result = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('result::result.name')) 
            ->view('result::result.create', true) 
            ->data(compact('result'))
            ->output();
    }

    /**
     * Create new result.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ResultRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $result                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('result::result.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('result/result/' . $result->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/result/result'))
                ->redirect();
        }

    }

    /**
     * Show result for editing.
     *
     * @param Request $request
     * @param Model   $result
     *
     * @return Response
     */
    public function edit(ResultRequest $request, Result $result)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('result::result.name'))
            ->view('result::result.edit', true)
            ->data(compact('result'))
            ->output();
    }

    /**
     * Update the result.
     *
     * @param Request $request
     * @param Model   $result
     *
     * @return Response
     */
    public function update(ResultRequest $request, Result $result)
    {
        try {
            $attributes = $request->all();

            $result->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('result::result.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('result/result/' . $result->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('result/result/' . $result->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the result.
     *
     * @param Model   $result
     *
     * @return Response
     */
    public function destroy(ResultRequest $request, Result $result)
    {
        try {

            $result->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('result::result.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('result/result/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('result/result/' . $result->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple result.
     *
     * @param Model   $result
     *
     * @return Response
     */
    public function delete(ResultRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('result::result.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('result/result'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/result/result'))
                ->redirect();
        }

    }

    /**
     * Restore deleted results.
     *
     * @param Model   $result
     *
     * @return Response
     */
    public function restore(ResultRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('result::result.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/result/result'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/result/result/'))
                ->redirect();
        }

    }

}
